<?php namespace bff\utils;

/**
 * Класс для работы с директориями / файлами
 * @abstract
 * @version 0.5
 * @modified 29.jan.2014
 */

abstract class Files
{
    /**
     * Формирование списка директорий относительно корневой директории
     * @param string $sPath путь к корневой директории
     * @return array
     */
    public static function getDirs($sPath)
    {
        $aResult = array();
        foreach (new \DirectoryIterator($sPath) as $file) {
            if ($file->isDir() && !$file->isDot()) {
                $aResult[] = $file->getFilename();
            }
        }

        return $aResult;
    }

    /**
     * Подсчет размера файлов(поддиректорий) в директории
     * @param mixed $sPath путь к директории
     * @return integer
     */
    public static function getDirSize($sPath)
    {
        $size = 0;
        foreach (new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($sPath)) as $file) {
            $size += $file->getSize();
        }

        return $size;
    }

    /**
     * Рекурсивный обход директорий
     * @param string $directory корневая директория
     * @param string $base путь относительно корневой директории
     * @param boolean $bRecursive рекурсивный обход
     * @param boolean $bReturnFullPath true - возвращать полный путь, false - только имя файла/директории
     * @param array $aFileTypes типы файлов
     * @param array $aExclude список исключений
     * @return array
     */
    public static function getFiles($directory, $base = '', $bRecursive = true, $bReturnFullPath = true, array $aFileTypes = array(), array $aExclude = array())
    {
        if (is_file($directory) || !is_dir($directory)) {
            return false;
        }

        $directory = rtrim($directory, '\\/');
        if (!is_dir($directory)) {
            return false;
        }

        # Открываем директорию
        if ($dir = opendir($directory)) {
            # Формируем список найденных файлов
            $tmp = Array();
            # Добавляем файлы
            while ($file = readdir($dir)) {
                if ($file === '.' || $file === '..')
                    continue;

                $isFile = is_file($directory . '/' . $file);
                if (!static::validatePath($base, $file, $isFile, $aFileTypes, $aExclude))
                    continue;

                if ($isFile) {
                    array_push($tmp, ($bReturnFullPath ? $directory . '/' . $file : $file));
                } else {
                    # Если директория > ищем в ней
                    if ($bRecursive) {
                        $tmpSub = static::getFiles($directory . '/' . $file, $base . '/' . $file, $bRecursive, $bReturnFullPath, $aFileTypes, $aExclude);
                        if (!empty($tmpSub)) {
                            $tmp = array_merge($tmp, $tmpSub);
                        }
                    }
                }
            }
            closedir($dir);

            return $tmp;
        }

        return array();
    }

    /**
     * Валидация пути файла / директории
     * @param string $base путь относительно корневой директории
     * @param string $file имя файла / директории
     * @param boolean $bIsFile
     * @param array $aFileTypes список суфиксов типов файлов (без точки). Проходят только файлы с указанными суфиксами.
     * @param array $aExclude список исключений
     * @return boolean true - файл / директория валидны
     */
    protected static function validatePath($base, $file, $bIsFile, array $aFileTypes, array $aExclude)
    {
        foreach ($aExclude as $e) {
            if ($file === $e || strpos($base . '/' . $file, $e) === 0) {
                return false;
            }
        }
        if (!$bIsFile || empty($aFileTypes)) {
            return true;
        }
        if (($pos = strrpos($file, '.')) !== false) {
            $type = substr($file, $pos + 1);

            return in_array($type, $aFileTypes);
        } else {
            return false;
        }
    }

    /**
     * Получение содержимого файла в виде строки
     * @param string $sFilePath путь к файлу
     * @return string
     */
    public static function getFileContent($sFilePath)
    {
        return file_get_contents($sFilePath);
    }

    /**
     * Запись строки в файл
     * @param string $sFilePath путь к файлу
     * @param string $sContent данные
     * @return boolean
     */
    public static function putFileContent($sFilePath, $sContent)
    {
        $res = file_put_contents($sFilePath, $sContent);

        return ($res !== false);
    }

    /**
     * Достаточно ли прав на запись
     * @param string $sPath путь к файлу / директории
     * @param boolean $bTriggerError вызывать пользовательскую ошибку
     * @return boolean
     */
    public static function haveWriteAccess($sPath, $bTriggerError = false)
    {
        if (!is_writable($sPath) && !chmod($sPath, 775)) {
            if ($bTriggerError) {
                trigger_error(sprintf('Unable to write to "%s"', realpath((is_dir($sPath) ? $sPath : dirname($sPath)))));
            }

            return false;
        }

        return true;
    }

    /**
     * Чистим имя файла от запрещенных символов
     * @param string $sFileName имя файла
     * @param boolean $bRelativePath относительный путь (true)
     * @return string очищенное имя файла
     */
    public static function cleanFilename($sFileName, $bRelativePath = false)
    {
        $bad = array(
            '../',
            '<!--',
            '-->',
            '<',
            '>',
            "'",
            '"',
            '&',
            '$',
            '#',
            '{',
            '}',
            '[',
            ']',
            '=',
            ';',
            '?',
            "%20",
            "%22",
            "%3c", // <
            "%253c", // <
            "%3e", // >
            "%0e", // >
            "%28", // (
            "%29", // )
            "%2528", // (
            "%26", // &
            "%24", // $
            "%3f", // ?
            "%3b", // ;
            "%3d", // =
        );

        if (!$bRelativePath) {
            $bad[] = './';
            $bad[] = '/';
        }

        return stripslashes(str_replace($bad, '', $sFileName));
    }

    /**
     * Создание директории
     * @param string $sDirectoryName имя директории
     * @param int $nPermission права (@see chmod)
     * @return boolean
     **/
    public static function makeDir($sDirectoryName, $nPermission = 0775)
    {
        $parent = dirname($sDirectoryName);
        if (!static::haveWriteAccess($parent, true)) {
            return false;
        }

        return mkdir($sDirectoryName, $nPermission);
    }

    /**
     * Получение расширения файла (без точки)
     * @param string $sPath путь к файлу
     * @return string расширение без точки
     */
    public static function getExtension($sPath)
    {
        $res = mb_strtolower(pathinfo($sPath, PATHINFO_EXTENSION));

        return ($res == 'jpeg' ? 'jpg' : $res);
    }

    /**
     * Проверка, является ли файл изображением
     * @param mixed $sPath путь к файлу
     * @param boolean $bCheckExtension проверять расширение файла
     * @return boolean
     */
    public static function isImageFile($sPath, $bCheckExtension = false)
    {
        if ($bCheckExtension && !in_array(static::getExtension($sPath), array('gif', 'jpg', 'png')))
            return false;

        $imSize = getimagesize($sPath);
        if (empty($imSize)) return false;
        if (in_array($imSize[2], array(IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG))) {
            return true;
        }

        return false;
    }

}